#include "ADC.h"

#define adcBufCount	12

unsigned short ADC_ans=0;
int ADC_count=0;
volatile short RecBufADC[adcBufCount];

//*************************************************************************
//initADC
//
//Precondition: none
//
//Postcondition: ADC has been setup and dummy read performed.  
//
//*************************************************************************
void initADC(void){
	//DDRA &= ~(1<<PA0);
	ACSR=(1<<ACD); //turn off comparator
	ADCSRA|=(0<<ADPS2)|(0<<ADPS1)|(0<<ADPS0); //set prescalar bits.  011-> DIVIDE BY 8
	ADCSRA|=(1<<ADEN); //enable ADC
	ADMUX =(0<<REFS1)|(1<<REFS0); //01-Select AVCC external VREF, 00-AREF, Internal Vref turned off, 10-Internal 1.1V, 11-Interal 2.56V vref

	//Set sleep mode for ADC noise reduction
	SMCR &= ~((1<<SM2)|(1<<SM1));
	SMCR |= (1<<SM0); //set sleep mode to ADC reduction mode.

	//ADCSRA|=(1<<ADIE); //enable ADC interupt

	//Dummy read
	ADCSRA |= (1<<ADSC); //start conversion
	ADC_isBusy();
}

//*************************************************************************
//ADC_isBusy
//
//Precondition: ADC init called.
//
//Postcondition: Checks to see if ADC is busy or not.
//
//*************************************************************************
void ADC_isBusy(void){
	//If ADSC = 1 then conversion is in progress
	while(ADCSRA & (1<<ADSC)){
		_delay_us(1);
	};
}


//*************************************************************************
//ADC_selChan
//
//Precondition: ADC init called.
//
//Postcondition: selects the ADC chan.
//
//*************************************************************************
void ADC_selChan(unsigned char chan){
	ADMUX &= ~((1<<MUX2)|(1<<MUX1)|(1<<MUX0)); //sets mux to chan zero by default
	ADMUX |= (chan & 0x07); //mask off the last three bits of chan.  
}


//*************************************************************************
//ADC_readChan
//
//Precondition: ADC init called.
//
//Postcondition:Reads the ADC channel.
//
//*************************************************************************
unsigned short ADC_readChan(unsigned char chan){
	unsigned short ans=0;
			
	ADC_selChan(chan);
	ADCSRA |= (1<<ADSC); //start conversion

	ADC_isBusy();

	ans = ADCL;
	ans |= (ADCH<<8);

	return ans;
}

//*************************************************************************
//ADC_readChan_IRQ
//
//Precondition: ADC init called.
//
//Postcondition:Reads the ADC channel.  This function is for use with the
//				Interrupt routine
//
//*************************************************************************
void ADC_readChan_IRQ(unsigned char chan){
			
	ADC_selChan(chan);
	ADCSRA |= (1<<ADSC); //start conversion

	ADC_isBusy();

}

//*****ADC Interupt
/*ISR(ADC_vect) 
{ 
   // Code to be executed when the ADC complete interrupt issued
   ADC_ans = ADCL;
   ADC_ans |= (ADCH<<8);
   ADCSRA |=(1<<ADIF);
  
}*/


