#ifndef PGLINK2_H
#define PGLINK2_H

//********************************************************************************************
//PGLink2 header file
//
//********************************************************************************************

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdlib.h>
#include "h2a2h.h"
#include "UARTDriver_644.h"
#include "ADC.h"

#ifndef F_CPU
	#define F_CPU 7372800UL
#endif

#define _enable_interrupt sei
#define _disable_interrupt cli

#define remoteBufCount	80

#define REMOTE	0
#define OFF		0
#define ON		1

#define PG_PORT 	PORTC
#define PG_DIRPORT 	DDRC
#define PG_READ		PINC
#define PG_CLOCK 	PC1
#define PG_LATCH	PC0
#define PG_DATA		PC2

#define	GLATCH      0x01    // PG latch out high
#define	GCLOCK      0x02    // PG clock out high
#define	GCLOLAT     0x03    // clock + latch high
#define GNONE		0x00    // both low

//PG Glove timing numbers
#define	  D2WIDTH 	16.8		//DEFUALT of 19us. This along with D2BITS define clock period. 
							    //85kbaud was nintendo data rate so minimum should be 11.77us
#define   D2LATCH 	5			//DEFUALT of 5us. Latch pulse width. Should be 3us to 8us wide
#define   D2BYTES 	76			//DEFAULT of 96us. Delay between last bit of last byte and latch.
								//can be brought as low as 40us but glove doesn't beep at bootup 
								//like it should.  Who knows what other issues are present.
#define   D2BITS  	3         	//DEFAULT of 3us. Clock low period which is what triggers 
								//next data byte to come out of glove after an initial latch pulse
#define   D2SLOW  	40	  	    //Timing sheet says 14.720ms. Really dependent on your loop timing
#define	  DCLK2LAT 	6			//Default of 6us. Setup time for latch in Hi Res function.

//Glove data structure for convience. Used mostly for PGlove.zip, THEGLOVE.C functions
typedef struct glove_data {
			  	unsigned char header,x,y,z,rot,fingers,keys,zero1,zero2,occ,f1,f2;
			  } glove_data;

//****************************************************************************************
//PROTOTYPES
//****************************************************************************************
//SYSTEM Level 
void setupHardware(void);
void setupUART(void);
void flushBuf(volatile unsigned char *array, int len);
void Hi_Res(void);
unsigned char getGlove(void);
unsigned char getByte(void);
unsigned char printPG(glove_data *g);
unsigned char pg_reset(void);
unsigned char printPGBinary(glove_data *g);
void printMENU(void);
//filtering functions from PGlove.zip
void dehyst(glove_data *g);
void deglitch(glove_data *g);


//Command messages
#define HELP    (RecBufREMOTE[0]=='H' && RecBufREMOTE[1]=='E' && RecBufREMOTE[2]=='L' && RecBufREMOTE[3]=='P')
#define TEST	(RecBufREMOTE[0]=='T' && RecBufREMOTE[1]=='E' && RecBufREMOTE[2]=='S' && RecBufREMOTE[3]=='T')
#define VER 	(RecBufREMOTE[0]=='V' && RecBufREMOTE[1]=='E' && RecBufREMOTE[2]=='R')
#define RESET	(RecBufREMOTE[0]=='R' && RecBufREMOTE[1]=='E' && RecBufREMOTE[2]=='S' && RecBufREMOTE[3]=='E' && RecBufREMOTE[4]=='T')
#define CONT	(RecBufREMOTE[0]=='C' && RecBufREMOTE[1]=='O' && RecBufREMOTE[2]=='N' && RecBufREMOTE[3]=='T')
#define HI_RES	(RecBufREMOTE[0]=='H' && RecBufREMOTE[1]=='I' && RecBufREMOTE[2]=='_' && RecBufREMOTE[3]=='R' && RecBufREMOTE[4]=='E' && RecBufREMOTE[5]=='S')
#define BIN 	(RecBufREMOTE[0]=='B' && RecBufREMOTE[1]=='I' && RecBufREMOTE[2]=='N')
#define XHYST	(RecBufREMOTE[0]=='X' && RecBufREMOTE[1]=='H' && RecBufREMOTE[2]=='Y' && RecBufREMOTE[3]=='S' && RecBufREMOTE[4]=='T')
#define YHYST	(RecBufREMOTE[0]=='Y' && RecBufREMOTE[1]=='H' && RecBufREMOTE[2]=='Y' && RecBufREMOTE[3]=='S' && RecBufREMOTE[4]=='T')
#define XACC	(RecBufREMOTE[0]=='X' && RecBufREMOTE[1]=='A' && RecBufREMOTE[2]=='C' && RecBufREMOTE[3]=='C')
#define YACC	(RecBufREMOTE[0]=='Y' && RecBufREMOTE[1]=='A' && RecBufREMOTE[2]=='C' && RecBufREMOTE[3]=='C')
#define XXTEND	(RecBufREMOTE[0]=='X' && RecBufREMOTE[1]=='X' && RecBufREMOTE[2]=='T' && RecBufREMOTE[3]=='E' && RecBufREMOTE[4]=='N' && RecBufREMOTE[5]=='D')
#define YXTEND	(RecBufREMOTE[0]=='Y' && RecBufREMOTE[1]=='X' && RecBufREMOTE[2]=='T' && RecBufREMOTE[3]=='E' && RecBufREMOTE[4]=='N' && RecBufREMOTE[5]=='D')

#endif
